
require "open-uri"
require "json"

module Inconcert

  module GenericHelpers

    #
    # Creates the correct url for the apt source.
    #
    def self.packages_repo_url(node)
      ps = node.inconcert.sources
      "#{ps['protocol']}://#{ps['server']}/#{ps['repo']}/ #{ps['module']}/"
    end

    #
    # Takes a hash representing the service data and returns the subdomain plus a dot or an empty string.
    # Removes any starting dashes in the subdomain
    #
    def self.subdomainOrNot( service )
      (((service["subdomain"] || "" ).chars.to_a << "-").drop_while{ |c| c == "-" }.join ) || ""
    end

  end

  module ServiceHelpers

    # Stupid forwarding. Allows this method to be used inside templates (like i6host.conf.erb)
    def subdomainOrNot (service)
      GenericHelpers::subdomainOrNot(service)
    end

    #
    # Correctly build the node's private host name
    #
    def private_self_host

      ip    = node.ipaddress
      host  = node.inconcert.nodes.select{ |n| n["id"] == node.hostname }

      ip    = host.first["private_ip"] if host.size > 0

      "#{ip} #{node.hostname}.#{node.inconcert.virtual_name} #{node.hostname}"

    end
       
    #
    # Lista de hosts a los que este nodo responde o conoce
    #
    def hosts_list
      #node.hostname << " " << node.inconcert.services.map do |s|
      node.inconcert.services.map do |s|
        "#{GenericHelpers::subdomainOrNot s}#{node.inconcert.virtual_name}"
      end.join(" ")
    end

    #
    # Returns a list of services filtered by the type given.
    #
    # This function is used to generate nginx upstreams.
    #
    def services(type)
      node.inconcert.services.select do |n|
        if n['type'].is_a? String then
          n['type'] == type
        elsif n['type'].respond_to? :include? then
          n['type'].include? type
        elsif n['type'].respond_to? :to_s then
          n['type'].to_s == type
        end
      end
    end
    
    #
    # Returns a service matching given name or nil.
    #
    def service_by_name(name)
      node.inconcert.services.detect do |n|
        # puts "#{name} == #{n['name']} -> #{name == n["name"]}"
        n["name"] == name
      end
    end

    #
    # Get attribute from service or nil of no service or attribute.
    #
    def service_attr(service,attr)
      svc = service_by_name(service)
      if svc != nil then
        svc[attr]
      else
        nil
      end
    end
    
    #
    # Returns only the services that match the type given and are
    # toplevel (have subdomain).
    #
    def toplevel_services(type)
      node.inconcert.services.select do |s|
        s["type"] == type && (s["force_toplevel"] || (s['subdomain'] != nil && s['subdomain'] != ""))
      end
    end

    #
    # Returns only the nodes that match the type given.
    #
    def nodes(type)
      node.inconcert.nodes.select do |n|
        if n['type'].is_a? String then
          n['type'] == type
        elsif n['type'].respond_to? :include? then
          n['type'].include? type
        elsif n['type'].respond_to? :to_s then
          n['type'].to_s == type
        end
      end
    end

    #
    # Find the index for a node or return nil if not found.
    #
    def node_index(id)
      (node.inconcert.nodes.find_index { |n| n["id"] == id }) + 1
    end


    #
    # Returns true if there is a monitor defined for this cluster.
    # Note that this does not mean that it is already working or even installed.
    #
    def monitor_defined?
      puts "MONITOR NODES -> #{nodes("monitor")}"
      nodes("monitor").size > 0
    end

    #
    # Tests whether the monitor is online or not by trying to get a token from the login app.
    #
    def monitor_online?
      if !monitor_defined? then
        puts "No monitor node defined"
        false
      else
        # FIXME lookup the monitors domain.
        host = nodes("monitor").first['private_ip']
        port = services("monitor").first["port"]
        puts("http://#{host}:#{port}/inconcert/apps/login/getToken")
        u = URI.parse("http://#{host}:#{port}/inconcert/apps/login/getToken")
        begin
          c = u.read
          true
        rescue => e
          puts e
          false
        end
      end
    end

  end

end
